#!/bin/bash
###############################################################################
# This library "prepare-system-image-for-read-only-mode" helps to configure 
# an image of Raspberry Pi OS or other compatible Linux distributions 
# to run in read-only mode.
#
# This library is supplementary material to the article 
# "How to configure Raspberry Pi OS to run in read-only mode". 
# The article and this library can be found at https://altomaxtech.com/. 
#
# Copyright (C) 2021, Vladimir Zlatov, Evgueni Tzvetanov, USA 
# Email us at support@altomaxtech.com. 
# 
# This library is free software; you can redistribute it 
# and/or modify it under the terms of the GNU Lesser General Public 
# License as published by the Free Software Foundation; either 
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# See the GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General 
# Public License along with this library; 
# if not, write to the Free Software Foundation, Inc., 
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
###############################################################################
###############################################################################
# This script was generated automatically by 
# 'prepare-system-image-for-read-only-mode.sh' 
# at Sun 08 Aug 2021 11:05:58 AM UTC.
###############################################################################
###############################################################################
# __RUN_COUNTER__=0
###################
update_run_counter() { 
    # $1 - increment the counter by value

    local key="__RUN_COUNTER__";
    local key_rex="^#\s\+$key=";
    local value_rex="[0-9]\+";
    local eol_rex=".*$";
    local line_rex="$key_rex$value_rex$eol_rex";
    local line=$(grep -e "$line_rex" $0); 
    if [ -n "$line" ] ; then
        local current_count=$(echo "$line" | cut -f2 -d= | sed 's/[^0-9]*//g'); 
        if [ "$1" -gt "0" ] ; then
            local new_count=$((current_count+$1)); 
            sed -i -e "/$line_rex/s/\($key_rex\)$value_rex\($eol_rex\)/\1$new_count\2/" $0; 
        fi
        # return current count value
        echo $current_count;
    else
        # return empty string
        echo "";
    fi
}

check_if_run_allowed__prompt_user_in_doubt__update_counter_on_yes_if_requested() { 
    # $1 - increment the counter by value
    local current_count=$(update_run_counter 0);
    if [ -n "$current_count" ] ; then
        if [ "$current_count" -gt "0" ] ; then
            read -p "This script has already run $current_count time(s). Do you really want to run it one more time? [yes[/]] "  answer
            local yes_pat="^[Yy][Ee][Ss]$";
            if [[ $answer =~ $yes_pat ]] ; then
                if [ "$1" -gt "0" ] ; then
                    update_run_counter "";
                fi
                return 0;
            fi
            
            return 1;
        fi
    fi
    return 0;
}
    
    


is_root() { return $(ls /root/ &>/dev/null); }

if ! is_root; then
    echo "Please run this script as root (use sudo)";
    exit 1;
fi


if ! check_if_run_allowed__prompt_user_in_doubt__update_counter_on_yes_if_requested 0; then
    exit 1;
fi


is_system_mounted_as_rw() {
    # returns  TRUE if none of '/' and '/boot/' found out to be 'ro'.
    # returns  FALSE otherwise.
    return $( ! findmnt -lo target,options | grep -e '^/\s\+ro' -e '^/boot.*\s\+ro' &>/dev/null);
}

is_system_root_mounted_as_rw() {
    # returns  TRUE if '/' is 'rw'.
    # returns  FALSE otherwise.
    return $( findmnt -lo target,options | grep -e '^/\s\+rw' &>/dev/null);
}

is_system_boot_mounted_as_rw() {
    # returns  TRUE if '/boot/' is 'rw'.
    # returns  FALSE otherwise.
    return $( findmnt -lo target,options | grep -e '^/boot.*\s\+rw' &>/dev/null);
}



onboard_create_report() {
    # $1 - command
    # $2 - command nickname
    # $3 - output directory (without closing slash)
    # $4 - output file name prefix
    # $5 - output file name postfix
    # $6 - owner name
    # $7 - Optional. A string to output instead of the command () report.
    # Returns:
    #       echo "$errors"

    local errors="";
    local path="$3/$4_$2_$5_report.txt";

    local true_str="TRUE";
    local ok="";


    if [[ -n $7 ]] ; then
        if echo "$7" > $path; then
            ok=$true_str;
        fi
    else
        if $1 > $path; then
            ok=$true_str;
        fi
    fi

    if [[ -n $ok ]] ; then
        if( ! chown $6:$6 $path ) ; then
            errors+="$2-chown;";
        fi
        if( ! chmod 644 $path ) ; then
            errors+="$2-chmod;";
        fi
    else
        errors+="$2;";
    fi    

    echo $errors;
}


escape_special_symbols_for_sed() {
    # $1 -- input string
    # Escapes symbols: '\', '(', ')', '?', '+'

    echo "$1" | sed -e 's/(/\\(/g' -e 's/)/\\)/g' -e 's/?/\\?/g' -e 's/+/\\+/g' -e 's/\//\\\//g' -e 's/|/\\|/g';
}

get_sed_replacement_string() {
    # $1 -- mountpoint
    # $2 -- fstype
    # $3 -- requested system mode: 'ro' or 'rw'.

    if [[ $3 = "ro" ]] ; then
        if [[ $1 =~ "/boot" ]]; then
            echo ",ro,noatime";
        else
            echo ",ro,noatime,noload";
        fi
    elif  [[ $3 = "rw" ]] ; then
        # "rw" mode
        if [[ $1 =~ "/boot" ]]; then
            echo "";    #",rw";
        else
            echo ",noatime";    #",rw,noatime";
        fi
    else 
        echo ",ERROR_UNKNOWN_SYSTEM_MODE_REQUESTED_$3";
    fi
}


    
remount_mountpoint() {
    # $1 -- mountpoint (For example, "/" - for root, "/boot" - for boot)
    # $2 -- mode to remount to ( "ro", "rw")

    local grep_filter="grep -w ^$1";
    local current_mount_info=$(findmnt -u -o TARGET,SOURCE,FSTYPE -D | $grep_filter);

    #https://www.gnu.org/software/sed/manual/sed.html
    local sed_search_param_1=$(escape_special_symbols_for_sed "^($1\S*)\s*.*$");
    local mountpoint=$(echo "$current_mount_info" | sed -e "s/$sed_search_param_1/\1/");

    local sed_search_param_2=$(escape_special_symbols_for_sed "^($1\S*)\s*(\S*)\s*.*$");
    local device=$(echo "$current_mount_info" | sed -e "s/$sed_search_param_2/\2/");
    
    local sed_search_param_3=$(escape_special_symbols_for_sed "^($1\S*)\s*(\S*)\s*(\S*)\s*.*$");
    local fstype=$(echo "$current_mount_info" | sed -e "s/$sed_search_param_3/\3/");

    local sed_replacement_string=$(get_sed_replacement_string "$mountpoint" "$fstype" "$2");

    local remount_options="remount,defaults";
    remount_options+=$(get_sed_replacement_string "$mountpoint" "$fstype" "$2");

    mount -o $remount_options $device $mountpoint ;
    #echo "ECHO: 'mount -o $remount_options $device $mountpoint ;'"
    return $?;

}

remount_boot_to_ro() {
    remount_mountpoint "/boot" "ro";
    return $?;
}

remount_boot_to_rw() {
    remount_mountpoint "/boot" "rw";
    return $?;
}

remount_root_to_ro() {
    remount_mountpoint "/" "ro";
    return $?;
}

remount_root_to_rw() {
    remount_mountpoint "/" "rw";
    return $?;
}

do_reports_and_reboot() {

    local error="";

    local old_is_ro="";
    if ! is_system_root_mounted_as_rw ; then
        old_is_ro="TRUE";
        if ! remount_root_to_rw ; then
            local error+="mount_rw;";
        fi
    fi

    error+=$(onboard_create_report "df -h" "df" "/home/pi" "21" "after_switching_to_read_only_mode" "pi");
    error+=$(onboard_create_report "free" "free" "/home/pi" "22" "after_switching_to_read_only_mode" "pi");
    error+=$(onboard_create_report "journalctl -b -p 0..3" "journal_b_p0_3" "/home/pi" "23" "after_switching_to_read_only_mode" "pi");
    error+=$(onboard_create_report "aplay -l" "aplay" "/home/pi" "24" "after_switching_to_read_only_mode" "pi");

    if ! ls -lsh /home/pi/2*; then
        error+="ls;";
    fi

    if [[ -z $error ]] ; then
        echo -e "\n\t\t\t\tReports for system running in read-only mode created SUCCESSFULLY. Reboot in 10 seconds...";
        update_run_counter 1;
        sleep 10;
        reboot;
    else
        echo "FAILED during report creation. The reports may be created OK. Hint for errors: '$error'.";
    fi
}

do_reports_and_reboot;


###############################################################################
###################  THE END  #################################################
###############################################################################
    

