#!/bin/bash
###############################################################################
# This library "prepare-system-image-for-read-only-mode" helps to configure 
# an image of Raspberry Pi OS or other compatible Linux distributions 
# to run in read-only mode.
#
# This library is supplementary material to the article 
# "How to configure Raspberry Pi OS to run in read-only mode". 
# The article and this library can be found at https://altomaxtech.com/. 
#
# Copyright (C) 2021, Vladimir Zlatov, Evgueni Tzvetanov, USA 
# Email us at support@altomaxtech.com. 
# 
# This library is free software; you can redistribute it 
# and/or modify it under the terms of the GNU Lesser General Public 
# License as published by the Free Software Foundation; either 
# version 2.1 of the License, or (at your option) any later version.
# 
# This library is distributed in the hope that it will be useful, 
# but WITHOUT ANY WARRANTY; without even the implied warranty of 
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
# See the GNU Lesser General Public License for more details.
# 
# You should have received a copy of the GNU Lesser General 
# Public License along with this library; 
# if not, write to the Free Software Foundation, Inc., 
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
###############################################################################
###############################################################################
# This script was generated automatically by 
# 'prepare-system-image-for-read-only-mode.sh' 
# at Sun 08 Aug 2021 11:05:58 AM UTC.
###############################################################################


is_system_mounted_as_rw() {
    # returns  TRUE if none of '/' and '/boot/' found out to be 'ro'.
    # returns  FALSE otherwise.
    return $( ! findmnt -lo target,options | grep -e '^/\s\+ro' -e '^/boot.*\s\+ro' &>/dev/null);
}

is_system_root_mounted_as_rw() {
    # returns  TRUE if '/' is 'rw'.
    # returns  FALSE otherwise.
    return $( findmnt -lo target,options | grep -e '^/\s\+rw' &>/dev/null);
}

is_system_boot_mounted_as_rw() {
    # returns  TRUE if '/boot/' is 'rw'.
    # returns  FALSE otherwise.
    return $( findmnt -lo target,options | grep -e '^/boot.*\s\+rw' &>/dev/null);
}


is_root() { return $(ls /root/ &>/dev/null); }

if ! is_root; then
    echo "Please run this script as root (use sudo)";
    exit 1;
fi


escape_special_symbols_for_sed() {
    # $1 -- input string
    # Escapes symbols: '\', '(', ')', '?', '+'

    echo "$1" | sed -e 's/(/\\(/g' -e 's/)/\\)/g' -e 's/?/\\?/g' -e 's/+/\\+/g' -e 's/\//\\\//g' -e 's/|/\\|/g';
}

get_sed_replacement_string() {
    # $1 -- mountpoint
    # $2 -- fstype
    # $3 -- requested system mode: 'ro' or 'rw'.

    if [[ $3 = "ro" ]] ; then
        if [[ $1 =~ "/boot" ]]; then
            echo ",ro,noatime";
        else
            echo ",ro,noatime,noload";
        fi
    elif  [[ $3 = "rw" ]] ; then
        # "rw" mode
        if [[ $1 =~ "/boot" ]]; then
            echo "";    #",rw";
        else
            echo ",noatime";    #",rw,noatime";
        fi
    else 
        echo ",ERROR_UNKNOWN_SYSTEM_MODE_REQUESTED_$3";
    fi
}


    
remount_mountpoint() {
    # $1 -- mountpoint (For example, "/" - for root, "/boot" - for boot)
    # $2 -- mode to remount to ( "ro", "rw")

    local grep_filter="grep -w ^$1";
    local current_mount_info=$(findmnt -u -o TARGET,SOURCE,FSTYPE -D | $grep_filter);

    #https://www.gnu.org/software/sed/manual/sed.html
    local sed_search_param_1=$(escape_special_symbols_for_sed "^($1\S*)\s*.*$");
    local mountpoint=$(echo "$current_mount_info" | sed -e "s/$sed_search_param_1/\1/");

    local sed_search_param_2=$(escape_special_symbols_for_sed "^($1\S*)\s*(\S*)\s*.*$");
    local device=$(echo "$current_mount_info" | sed -e "s/$sed_search_param_2/\2/");
    
    local sed_search_param_3=$(escape_special_symbols_for_sed "^($1\S*)\s*(\S*)\s*(\S*)\s*.*$");
    local fstype=$(echo "$current_mount_info" | sed -e "s/$sed_search_param_3/\3/");

    local sed_replacement_string=$(get_sed_replacement_string "$mountpoint" "$fstype" "$2");

    local remount_options="remount,defaults";
    remount_options+=$(get_sed_replacement_string "$mountpoint" "$fstype" "$2");

    mount -o $remount_options $device $mountpoint ;
    #echo "ECHO: 'mount -o $remount_options $device $mountpoint ;'"
    return $?;

}

remount_boot_to_ro() {
    remount_mountpoint "/boot" "ro";
    return $?;
}

remount_boot_to_rw() {
    remount_mountpoint "/boot" "rw";
    return $?;
}

remount_root_to_ro() {
    remount_mountpoint "/" "ro";
    return $?;
}

remount_root_to_rw() {
    remount_mountpoint "/" "rw";
    return $?;
}

old_is_ro="";
if ! is_system_root_mounted_as_rw ; then
    old_is_ro="TRUE";
    if ! remount_root_to_rw ; then
        echo "Cannot remount root to 'rw' mode.";
        exit 1;
    fi
fi


if [[ -e /etc/fstab.bak0 ]] ; then
    cp /etc/fstab /etc/fstab.bak;
else
    cp /etc/fstab /etc/fstab.bak0;
fi
sed -i -e 's/\(^PARTUUID=\S\+\s\+\/boot\s\+vfat\s\+\)\(defaults\)\+\(,\)\?\(r[wo]\|noatime\|noload\)\?\(,\)\?\(r[wo]\|noatime\|noload\)\?\(,\)\?\(r[wo]\|noatime\|noload\)\?\(.*$\)/\1\2\9/' -e 's/\(^PARTUUID=\S\+\s\+\/\s\+ext4\s\+\)\(defaults\)\+\(,\)\?\(r[wo]\|noatime\|noload\)\?\(,\)\?\(r[wo]\|noatime\|noload\)\?\(,\)\?\(r[wo]\|noatime\|noload\)\?\(.*$\)/\1\2,noatime\9/'  /etc/fstab;

echo "The system has been switched to the 'rw' boot mode.";
echo "Note that the changes will only take effect after reboot.";

if [[ -n $old_is_ro ]] ; then
    if ! remount_root_to_ro ; then
        echo "Cannot remount root back to 'ro' mode.";
    fi
fi


###############################################################################
###################  THE END  #################################################
###############################################################################
    

